pub struct Triangle<T = f64>(pub Coord<T>, pub Coord<T>, pub Coord<T>)
where
T: CoordNum;Expand description
A bounded 2D area whose three vertices are defined by
Coords. The semantics and validity are that of
the equivalent Polygon; in addition, the three
vertices must not be collinear and they must be distinct.
Notes
Irrespective of input order the resulting geometry has ccw order and its vertices are yielded in ccw order by iterators
Tuple Fields
0: Coord<T>1: Coord<T>2: Coord<T>Implementations
Sourceimpl<T> Triangle<T>where
T: CoordNum,
impl<T> Triangle<T>where
T: CoordNum,
Sourcepub fn new(v1: Coord<T>, v2: Coord<T>, v3: Coord<T>) -> Triangle<T>
pub fn new(v1: Coord<T>, v2: Coord<T>, v3: Coord<T>) -> Triangle<T>
Create a new Triangle, enforcing default (counter clockwise) winding order.
If the vertices are given in clockwise order, they will be reversed. Degenerate triangles (collinear or identical vertices) are stored as-is.
Sourcepub fn unchecked_winding(
v1: Coord<T>,
v2: Coord<T>,
v3: Coord<T>,
) -> Triangle<T>
pub fn unchecked_winding( v1: Coord<T>, v2: Coord<T>, v3: Coord<T>, ) -> Triangle<T>
Create a Triangle without normalising the winding order.
Use this when the caller has already verified CCW winding, or when the calling code does not depend on a particular winding order (e.g. inside tight loops where the orientation check would be wasteful).
Unlike Triangle::new, vertices are stored in the order given.
pub fn to_array(&self) -> [Coord<T>; 3]
pub fn to_lines(&self) -> [Line<T>; 3]
Sourcepub fn to_polygon(self) -> Polygon<T>
pub fn to_polygon(self) -> Polygon<T>
Create a Polygon from the Triangle.
Examples
use geo_types::{coord, Triangle, polygon};
// Input is CW
let triangle = Triangle::new(
coord! { x: 0., y: 0. },
coord! { x: 10., y: 20. },
coord! { x: 20., y: -10. },
);
// Output is CCW
assert_eq!(
triangle.to_polygon(),
polygon![
(x: 20., y: -10.),
(x: 10., y: 20.),
(x: 0., y: 0.),
(x: 20., y: -10.),
],
);Trait Implementations
Sourceimpl<T> AbsDiffEq for Triangle<T>
impl<T> AbsDiffEq for Triangle<T>
Sourcefn abs_diff_eq(
&self,
other: &Triangle<T>,
epsilon: <Triangle<T> as AbsDiffEq>::Epsilon,
) -> bool
fn abs_diff_eq( &self, other: &Triangle<T>, epsilon: <Triangle<T> as AbsDiffEq>::Epsilon, ) -> bool
Equality assertion with an absolute limit.
Examples
use geo_types::{point, Triangle};
let a = Triangle::new((0.0, 0.0).into(), (10.0, 10.0).into(), (0.0, 5.0).into());
let b = Triangle::new((0.0, 0.0).into(), (10.01, 10.0).into(), (0.0, 5.0).into());
approx::abs_diff_eq!(a, b, epsilon=0.1);
approx::abs_diff_ne!(a, b, epsilon=0.001);Sourcefn default_epsilon() -> <Triangle<T> as AbsDiffEq>::Epsilon
fn default_epsilon() -> <Triangle<T> as AbsDiffEq>::Epsilon
Sourcefn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
fn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
AbsDiffEq::abs_diff_eq.Sourceimpl<T> Area<T> for Triangle<T>where
T: CoordFloat,
impl<T> Area<T> for Triangle<T>where
T: CoordFloat,
fn signed_area(&self) -> T
fn unsigned_area(&self) -> T
Sourceimpl<T> BoundingRect<T> for Triangle<T>where
T: CoordNum,
impl<T> BoundingRect<T> for Triangle<T>where
T: CoordNum,
Sourceimpl<T> Centroid for Triangle<T>where
T: GeoFloat,
impl<T> Centroid for Triangle<T>where
T: GeoFloat,
Sourceimpl<T> ChamberlainDuquetteArea<T> for Triangle<T>where
T: CoordFloat,
impl<T> ChamberlainDuquetteArea<T> for Triangle<T>where
T: CoordFloat,
fn chamberlain_duquette_signed_area(&self) -> T
fn chamberlain_duquette_unsigned_area(&self) -> T
Sourceimpl<F: GeoFloat> ClosestPoint<F> for Triangle<F>
impl<F: GeoFloat> ClosestPoint<F> for Triangle<F>
Sourcefn closest_point(&self, p: &Point<F>) -> Closest<F>
fn closest_point(&self, p: &Point<F>) -> Closest<F>
self and p.Sourceimpl<T> Contains<GeometryCollection<T>> for Triangle<T>where
T: GeoFloat,
impl<T> Contains<GeometryCollection<T>> for Triangle<T>where
T: GeoFloat,
fn contains(&self, target: &GeometryCollection<T>) -> bool
Sourceimpl<T> Contains<LineString<T>> for Triangle<T>where
T: GeoFloat,
impl<T> Contains<LineString<T>> for Triangle<T>where
T: GeoFloat,
fn contains(&self, target: &LineString<T>) -> bool
Sourceimpl<T> Contains<MultiLineString<T>> for Triangle<T>where
T: GeoFloat,
impl<T> Contains<MultiLineString<T>> for Triangle<T>where
T: GeoFloat,
fn contains(&self, target: &MultiLineString<T>) -> bool
Sourceimpl<T> Contains<MultiPoint<T>> for Triangle<T>where
T: GeoFloat,
impl<T> Contains<MultiPoint<T>> for Triangle<T>where
T: GeoFloat,
fn contains(&self, target: &MultiPoint<T>) -> bool
Sourceimpl<T> Contains<MultiPolygon<T>> for Triangle<T>where
T: GeoFloat,
impl<T> Contains<MultiPolygon<T>> for Triangle<T>where
T: GeoFloat,
fn contains(&self, target: &MultiPolygon<T>) -> bool
Sourceimpl<T> CoordinatePosition for Triangle<T>where
T: GeoNum,
impl<T> CoordinatePosition for Triangle<T>where
T: GeoNum,
Sourceimpl<'a, T: CoordNum + 'a> CoordsIter<'a> for Triangle<T>
impl<'a, T: CoordNum + 'a> CoordsIter<'a> for Triangle<T>
Sourcefn coords_count(&'a self) -> usize
fn coords_count(&'a self) -> usize
Return the number of coordinates in the Triangle.
type Iter = Chain<Chain<Once<Coord<T>>, Once<Coord<T>>>, Once<Coord<T>>>
type ExteriorIter = <Triangle<T> as CoordsIter<'a>>::Iter
type Scalar = T
Sourcefn coords_iter(&'a self) -> Self::Iter
fn coords_iter(&'a self) -> Self::Iter
Sourcefn exterior_coords_iter(&'a self) -> Self::ExteriorIter
fn exterior_coords_iter(&'a self) -> Self::ExteriorIter
Sourceimpl<T> EuclideanDistance<T, Point<T>> for Triangle<T>where
T: GeoFloat,
impl<T> EuclideanDistance<T, Point<T>> for Triangle<T>where
T: GeoFloat,
Sourcefn euclidean_distance(&self, point: &Point<T>) -> T
fn euclidean_distance(&self, point: &Point<T>) -> T
Sourceimpl GeodesicArea<f64> for Triangle
impl GeodesicArea<f64> for Triangle
Sourcefn geodesic_perimeter(&self) -> f64
fn geodesic_perimeter(&self) -> f64
Sourcefn geodesic_area_signed(&self) -> f64
fn geodesic_area_signed(&self) -> f64
Sourcefn geodesic_area_unsigned(&self) -> f64
fn geodesic_area_unsigned(&self) -> f64
Sourceimpl<C: GeoNum> HasDimensions for Triangle<C>
impl<C: GeoNum> HasDimensions for Triangle<C>
Sourcefn dimensions(&self) -> Dimensions
fn dimensions(&self) -> Dimensions
Rects are 2-dimensional, but it’s possible to create degenerate Rects which
have either 1 or 0 dimensions. Read moreSourcefn boundary_dimensions(&self) -> Dimensions
fn boundary_dimensions(&self) -> Dimensions
Geometry’s boundary, as used by OGC-SFA. Read moreSourceimpl<T> InteriorPoint for Triangle<T>where
T: GeoFloat,
impl<T> InteriorPoint for Triangle<T>where
T: GeoFloat,
Sourceimpl<T, G> Intersects<G> for Triangle<T>
impl<T, G> Intersects<G> for Triangle<T>
fn intersects(&self, rhs: &G) -> bool
Sourceimpl<T> Intersects<Triangle<T>> for Coord<T>
impl<T> Intersects<Triangle<T>> for Coord<T>
fn intersects(&self, rhs: &Triangle<T>) -> bool
Sourceimpl<T> Intersects<Triangle<T>> for Line<T>
impl<T> Intersects<Triangle<T>> for Line<T>
fn intersects(&self, rhs: &Triangle<T>) -> bool
Sourceimpl<T> Intersects<Triangle<T>> for Polygon<T>
impl<T> Intersects<Triangle<T>> for Polygon<T>
fn intersects(&self, rhs: &Triangle<T>) -> bool
Sourceimpl<T> Intersects<Triangle<T>> for Rect<T>
impl<T> Intersects<Triangle<T>> for Rect<T>
fn intersects(&self, rhs: &Triangle<T>) -> bool
Sourceimpl<T: CoordNum, NT: CoordNum> MapCoords<T, NT> for Triangle<T>
impl<T: CoordNum, NT: CoordNum> MapCoords<T, NT> for Triangle<T>
Sourceimpl<T: CoordNum> MapCoordsInPlace<T> for Triangle<T>
impl<T: CoordNum> MapCoordsInPlace<T> for Triangle<T>
Sourceimpl<T: CoordNum> MapCoordsInplace<T> for Triangle<T>
impl<T: CoordNum> MapCoordsInplace<T> for Triangle<T>
Sourcefn map_coords_inplace(&mut self, func: impl Fn((T, T)) -> (T, T) + Copy)where
T: CoordNum,
👎Deprecated since 0.21.0: use MapCoordsInPlace::map_coords_in_place instead which takes a Coord instead of an (x,y) tuple
fn map_coords_inplace(&mut self, func: impl Fn((T, T)) -> (T, T) + Copy)where
T: CoordNum,
MapCoordsInPlace::map_coords_in_place instead which takes a Coord instead of an (x,y) tupleApply a function to all the coordinates in a geometric object, in place
Examples
#[allow(deprecated)]
use geo::MapCoordsInplace;
use geo::Point;
use approx::assert_relative_eq;
let mut p = Point::new(10., 20.);
#[allow(deprecated)]
p.map_coords_inplace(|(x, y)| (x + 1000., y * 2.));
assert_relative_eq!(p, Point::new(1010., 40.), epsilon = 1e-6);Sourceimpl<T> RTreeObject for Triangle<T>
impl<T> RTreeObject for Triangle<T>
Sourceimpl<F: GeoFloat> Relate<F, GeometryCollection<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, GeometryCollection<F>> for Triangle<F>
fn relate(&self, other: &GeometryCollection<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Line<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, Line<F>> for Triangle<F>
fn relate(&self, other: &Line<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, LineString<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, LineString<F>> for Triangle<F>
fn relate(&self, other: &LineString<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, MultiLineString<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, MultiLineString<F>> for Triangle<F>
fn relate(&self, other: &MultiLineString<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, MultiPoint<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, MultiPoint<F>> for Triangle<F>
fn relate(&self, other: &MultiPoint<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, MultiPolygon<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, MultiPolygon<F>> for Triangle<F>
fn relate(&self, other: &MultiPolygon<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Point<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, Point<F>> for Triangle<F>
fn relate(&self, other: &Point<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Polygon<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, Polygon<F>> for Triangle<F>
fn relate(&self, other: &Polygon<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Rect<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, Rect<F>> for Triangle<F>
fn relate(&self, other: &Rect<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for GeometryCollection<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for GeometryCollection<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for Line<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for Line<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for LineString<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for LineString<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for MultiLineString<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for MultiLineString<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for MultiPoint<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for MultiPoint<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for MultiPolygon<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for MultiPolygon<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for Point<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for Point<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for Polygon<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for Polygon<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for Rect<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for Rect<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<F: GeoFloat> Relate<F, Triangle<F>> for Triangle<F>
impl<F: GeoFloat> Relate<F, Triangle<F>> for Triangle<F>
fn relate(&self, other: &Triangle<F>) -> IntersectionMatrix
Sourceimpl<T> RelativeEq for Triangle<T>where
T: CoordNum + RelativeEq<Epsilon = T>,
impl<T> RelativeEq for Triangle<T>where
T: CoordNum + RelativeEq<Epsilon = T>,
Sourcefn relative_eq(
&self,
other: &Triangle<T>,
epsilon: <Triangle<T> as AbsDiffEq>::Epsilon,
max_relative: <Triangle<T> as AbsDiffEq>::Epsilon,
) -> bool
fn relative_eq( &self, other: &Triangle<T>, epsilon: <Triangle<T> as AbsDiffEq>::Epsilon, max_relative: <Triangle<T> as AbsDiffEq>::Epsilon, ) -> bool
Equality assertion within a relative limit.
Examples
use geo_types::{point, Triangle};
let a = Triangle::new((0.0, 0.0).into(), (10.0, 10.0).into(), (0.0, 5.0).into());
let b = Triangle::new((0.0, 0.0).into(), (10.01, 10.0).into(), (0.0, 5.0).into());
approx::assert_relative_eq!(a, b, max_relative=0.1);
approx::assert_relative_ne!(a, b, max_relative=0.0001);Sourcefn default_max_relative() -> <Triangle<T> as AbsDiffEq>::Epsilon
fn default_max_relative() -> <Triangle<T> as AbsDiffEq>::Epsilon
Sourcefn relative_ne(
&self,
other: &Rhs,
epsilon: Self::Epsilon,
max_relative: Self::Epsilon,
) -> bool
fn relative_ne( &self, other: &Rhs, epsilon: Self::Epsilon, max_relative: Self::Epsilon, ) -> bool
RelativeEq::relative_eq.Sourceimpl<T> RemoveRepeatedPoints<T> for Triangle<T>where
T: CoordNum + FromPrimitive,
impl<T> RemoveRepeatedPoints<T> for Triangle<T>where
T: CoordNum + FromPrimitive,
Sourcefn remove_repeated_points(&self) -> Self
fn remove_repeated_points(&self) -> Self
Sourcefn remove_repeated_points_mut(&mut self)
fn remove_repeated_points_mut(&mut self)
Sourceimpl<T> TryFrom<Geometry<T>> for Triangle<T>where
T: CoordNum,
Convert a Geometry enum into its inner type.
impl<T> TryFrom<Geometry<T>> for Triangle<T>where
T: CoordNum,
Convert a Geometry enum into its inner type.
Fails if the enum case does not match the type you are trying to convert it to.
Sourceimpl<T: CoordNum, NT: CoordNum, E> TryMapCoords<T, NT, E> for Triangle<T>
impl<T: CoordNum, NT: CoordNum, E> TryMapCoords<T, NT, E> for Triangle<T>
Sourcetype Output = Triangle<NT>
type Output = Triangle<NT>
MapCoords::try_map_coords which takes a Coord instead of an (x,y) tupleSourcefn try_map_coords(
&self,
func: impl Fn((T, T)) -> Result<(NT, NT), E> + Copy,
) -> Result<Self::Output, E>
fn try_map_coords( &self, func: impl Fn((T, T)) -> Result<(NT, NT), E> + Copy, ) -> Result<Self::Output, E>
MapCoords::try_map_coords which takes a Coord instead of an (x,y) tupleSourceimpl<T: CoordNum, E> TryMapCoordsInplace<T, E> for Triangle<T>
impl<T: CoordNum, E> TryMapCoordsInplace<T, E> for Triangle<T>
Sourcefn try_map_coords_inplace(
&mut self,
func: impl Fn((T, T)) -> Result<(T, T), E>,
) -> Result<(), E>
fn try_map_coords_inplace( &mut self, func: impl Fn((T, T)) -> Result<(T, T), E>, ) -> Result<(), E>
MapCoordsInPlace::try_map_coords_in_place which takes a Coord instead of an (x,y) tupleResult. Read moreSourceimpl<T> UlpsEq for Triangle<T>
impl<T> UlpsEq for Triangle<T>
Sourcefn default_max_ulps() -> u32
fn default_max_ulps() -> u32
impl<T> Copy for Triangle<T>
impl<T> Eq for Triangle<T>
impl<T> StructuralPartialEq for Triangle<T>where
T: CoordNum,